@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_erf.c 1319 2008-01-02 21:10:07Z perry $
+ * $Id: format_erf.c 1332 2008-03-25 21:45:36Z spa1 $
  *
  */
 #define _GNU_SOURCE
@@ -124,13 +124,15 @@ static int erf_get_padding(const libtrace_packet_t *packet)
 	if (packet->trace->format->type==TRACE_FORMAT_ERF) {
 		dag_record_t *erfptr = (dag_record_t *)packet->header;
 		switch(erfptr->type) {
-			case TYPE_ETH: 		return 2;
+			case TYPE_ETH: 		
+			case TYPE_DSM_COLOR_ETH:
+				return 2;
 			default: 		return 0;
 		}
 	}
 	else {
 		switch(trace_get_link_type(packet)) {
-			case TYPE_ETH:		return 2;
+			case TRACE_TYPE_ETH:	return 2;
 			default:		return 0;
 		}
 	}
@@ -160,6 +162,13 @@ static int erf_config_input(libtrace_t *libtrace, trace_option_t option,
 		case TRACE_OPTION_EVENT_REALTIME:
 			IN_OPTIONS.real_time = *(int *)value;
 			return 0;
+		case TRACE_OPTION_SNAPLEN:
+		case TRACE_OPTION_PROMISC:
+		case TRACE_OPTION_FILTER:
+		case TRACE_OPTION_META_FREQ:
+			trace_set_err(libtrace, TRACE_ERR_OPTION_UNAVAIL,
+					"Unsupported option");
+			return -1;
 		default:
 			/* Unknown option */
 			trace_set_err(libtrace,TRACE_ERR_UNKNOWN_OPTION,
@@ -332,16 +341,63 @@ static int erf_fin_output(libtrace_out_t *libtrace) {
 	return 0;
 }
  
+static int erf_prepare_packet(libtrace_t *libtrace, libtrace_packet_t *packet,
+		void *buffer, libtrace_rt_types_t rt_type, uint32_t flags) {
+	
+	dag_record_t *erfptr;
+	
+	if (packet->buffer != buffer && 
+		packet->buf_control == TRACE_CTRL_PACKET) {
+		free(packet->buffer);
+	}
+
+	if ((flags & TRACE_PREP_OWN_BUFFER) == TRACE_PREP_OWN_BUFFER) {
+		packet->buf_control = TRACE_CTRL_PACKET;
+	} else
+	        packet->buf_control = TRACE_CTRL_EXTERNAL;
+	
+	
+	packet->type = rt_type;
+	packet->buffer = buffer;
+	packet->header = buffer;
+	erfptr = (dag_record_t *)packet->buffer;
+	if (erfptr->flags.rxerror == 1) {
+		packet->payload = NULL;
+		/* XXX: Do we want to do this? We never used to do this
+		 
+		erfptr->rlen = htons(erf_get_framing_length(packet));
+		*/
+	} else {
+		packet->payload = (char*)packet->buffer + erf_get_framing_length(packet);
+	}
+	
+	if (libtrace->format_data == NULL) {
+		/* Allocates the format_data structure */
+		if (erf_init_input(libtrace)) 
+			return -1;
+	}
+
+	/* Check for loss */
+	if (erfptr->type == TYPE_DSM_COLOR_ETH) {
+		/* No idea how we get this yet */
+
+	} else {
+		DATA(libtrace)->drops += ntohs(erfptr->lctr);
+	}
+
+	return 0;
+}
 
 static int erf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	int numbytes;
 	unsigned int size;
 	void *buffer2 = packet->buffer;
 	unsigned int rlen;
-
+	uint32_t flags = 0;
+	
+	
 	if (!packet->buffer || packet->buf_control == TRACE_CTRL_EXTERNAL) {
 		packet->buffer = malloc((size_t)LIBTRACE_PACKET_BUFSIZE);
-		packet->buf_control = TRACE_CTRL_PACKET;
 		if (!packet->buffer) {
 			trace_set_err(libtrace, errno, 
 					"Cannot allocate memory");
@@ -349,11 +405,8 @@ static int erf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		}
 	}
 
+	flags |= TRACE_PREP_OWN_BUFFER;	
 	
-	
-	packet->header = packet->buffer;
-	packet->type = TRACE_RT_DATA_ERF;
-
 	if ((numbytes=libtrace_io_read(INPUT.file,
 					packet->buffer,
 					(size_t)dag_record_size)) == -1) {
@@ -376,7 +429,7 @@ static int erf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	}
 
 	/* Unknown/corrupt */
-	if (((dag_record_t *)packet->buffer)->type >= 10) {
+	if (((dag_record_t *)packet->buffer)->type >= TYPE_AAL2) {
 		trace_set_err(libtrace, TRACE_ERR_BAD_PACKET, "Corrupt or Unknown ERF type");
 		return -1;
 	}
@@ -393,11 +446,10 @@ static int erf_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		/* Failed to read the full packet?  must be EOF */
 		return -1;
 	}
-	if (((dag_record_t *)packet->buffer)->flags.rxerror == 1) {
-		packet->payload = NULL;
-	} else {
-		packet->payload = (char*)packet->buffer + erf_get_framing_length(packet);
-	}
+	
+	if (erf_prepare_packet(libtrace, packet, packet->buffer, TRACE_RT_DATA_ERF, flags))
+		return -1;
+	
 	return rlen;
 }
 
@@ -599,6 +651,10 @@ size_t erf_set_capture_length(libtrace_packet_t *packet, size_t size) {
 		/* can't make a packet larger */
 		return trace_get_capture_length(packet);
 	}
+	/* Reset cached capture length - otherwise we will both return the
+	 * wrong value here and subsequent get_capture_length() calls will
+	 * return the wrong value. */
+	packet->capture_length = -1;
 	erfptr = (dag_record_t *)packet->header;
 	erfptr->rlen = htons(size + erf_get_framing_length(packet));
 	return trace_get_capture_length(packet);
@@ -623,11 +679,13 @@ static struct libtrace_eventobj_t erf_event(struct libtrace_t *libtrace, struct
 
 static uint64_t erf_get_dropped_packets(libtrace_t *trace)
 {
+	if (trace->format_data == NULL)
+		return (uint64_t)-1;
 	return DATA(trace)->drops;
 }
 
 static void erf_help(void) {
-	printf("erf format module: $Revision: 1319 $\n");
+	printf("erf format module: $Revision: 1332 $\n");
 	printf("Supported input URIs:\n");
 	printf("\terf:/path/to/file\t(uncompressed)\n");
 	printf("\terf:/path/to/file.gz\t(gzip-compressed)\n");
@@ -652,7 +710,7 @@ static void erf_help(void) {
 
 static struct libtrace_format_t erfformat = {
 	"erf",
-	"$Id: format_erf.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_erf.c 1332 2008-03-25 21:45:36Z spa1 $",
 	TRACE_FORMAT_ERF,
 	erf_init_input,			/* init_input */	
 	erf_config_input,		/* config_input */
@@ -664,6 +722,7 @@ static struct libtrace_format_t erfformat = {
 	erf_fin_input,			/* fin_input */
 	erf_fin_output,			/* fin_output */
 	erf_read_packet,		/* read_packet */
+	erf_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	erf_write_packet,		/* write_packet */
 	erf_get_link_type,		/* get_link_type */
