@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $
+ * $Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $
  *
  */
 #define _GNU_SOURCE
@@ -55,6 +55,10 @@
 #  define O_LARGEFILE 0
 #endif 
 
+#ifndef UINT32_MAX
+#  define UINT32_MAX 0xffffffff
+#endif
+
 #define DATA(x) ((struct legacy_format_data_t *)x->format_data)
 #define INPUT DATA(libtrace)->input
 
@@ -68,6 +72,15 @@ struct legacy_format_data_t {
 	uint32_t ts_old; 	/* Used for legacy_nzix */
 };
 
+static void legacy_init_format_data(libtrace_t *libtrace) {
+	libtrace->format_data = malloc(sizeof(struct legacy_format_data_t));
+	
+	DATA(libtrace)->input.file = NULL;
+	DATA(libtrace)->ts_high = 0;
+	DATA(libtrace)->ts_old = 0;
+	DATA(libtrace)->starttime = 0;
+}
+
 static int legacyeth_get_framing_length(const libtrace_packet_t *packet UNUSED) 
 {
 	return sizeof(legacy_ether_t);
@@ -90,9 +103,7 @@ static int legacynzix_get_framing_length(const libtrace_packet_t *packet UNUSED)
 
 static int erf_init_input(libtrace_t *libtrace) 
 {
-	libtrace->format_data = malloc(sizeof(struct legacy_format_data_t));
-
-	DATA(libtrace)->input.file = NULL;
+	legacy_init_format_data(libtrace);
 
 	return 0;
 }
@@ -135,10 +146,8 @@ static int legacynzix_init_input(libtrace_t *libtrace) {
 	regex_t reg;
 	regmatch_t match;
 
-	libtrace->format_data = malloc(sizeof(struct legacy_format_data_t));
 
-	DATA(libtrace)->input.file = NULL;
-	
+	legacy_init_format_data(libtrace);	
 	if((retval = regcomp(&reg, "[0-9]{8}-[0-9]{6}", REG_EXTENDED)) != 0) {
 		trace_set_err(libtrace, errno, "Failed to compile regex");
 		return -1;
@@ -148,8 +157,6 @@ static int legacynzix_init_input(libtrace_t *libtrace) {
 		return -1;
 	}
 	DATA(libtrace)->starttime = trtime(&filename[match.rm_so]);
-	DATA(libtrace)->ts_high = 0;
-	DATA(libtrace)->ts_old = 0;
 	return 0;
 }
 
@@ -172,14 +179,43 @@ static int erf_fin_input(libtrace_t *libtrace) {
 	return 0;
 }
 
+static int legacy_prepare_packet(libtrace_t *libtrace, 
+		libtrace_packet_t *packet, void *buffer, 
+		libtrace_rt_types_t rt_type, uint32_t flags) {
+
+	if (packet->buffer != buffer &&
+                        packet->buf_control == TRACE_CTRL_PACKET) {
+                free(packet->buffer);
+        }
+
+        if ((flags & TRACE_PREP_OWN_BUFFER) == TRACE_PREP_OWN_BUFFER) {
+                packet->buf_control = TRACE_CTRL_PACKET;
+        } else
+                packet->buf_control = TRACE_CTRL_EXTERNAL;
+
+
+        packet->buffer = buffer;
+        packet->header = buffer;
+	packet->type = rt_type;
+	packet->payload = (void*)((char*)packet->buffer + 
+		libtrace->format->get_framing_length(packet));
+
+
+	if (libtrace->format_data == NULL) {
+		legacy_init_format_data(libtrace);
+	}
+	return 0;
+}
+
 static int legacy_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	int numbytes;
 	void *buffer;
-
+	uint32_t flags = 0;
+	
 	if (!packet->buffer || packet->buf_control == TRACE_CTRL_EXTERNAL) {
-		packet->buf_control = TRACE_CTRL_PACKET;
 		packet->buffer=malloc((size_t)LIBTRACE_PACKET_BUFSIZE);
 	}
+	flags |= TRACE_PREP_OWN_BUFFER;
 	buffer = packet->buffer;
 
 	switch(libtrace->format->type) {
@@ -214,9 +250,10 @@ static int legacy_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		break;
 	}
 	
-	packet->header = packet->buffer;
-	packet->payload = (void*)((char*)packet->buffer + 
-		libtrace->format->get_framing_length(packet));
+	if (legacy_prepare_packet(libtrace, packet, packet->buffer, 
+				packet->type, flags)) {
+		return -1;
+	}
 	
 	return 64;
 	
@@ -230,13 +267,14 @@ static int legacynzix_read_packet(libtrace_t *libtrace, libtrace_packet_t *packe
 	int numbytes;
 	void *buffer;
 	char *data_ptr;
+	uint32_t flags = 0;
 	
 	if (!packet->buffer || packet->buf_control == TRACE_CTRL_EXTERNAL) {
-		packet->buf_control = TRACE_CTRL_PACKET;
 		packet->buffer=malloc((size_t)LIBTRACE_PACKET_BUFSIZE);
 	}
+	flags |= TRACE_PREP_OWN_BUFFER;
+	
 	buffer = packet->buffer;
-
 	packet->type = TRACE_RT_DATA_LEGACY_NZIX;
 	
 	while (1) {
@@ -262,22 +300,27 @@ static int legacynzix_read_packet(libtrace_t *libtrace, libtrace_packet_t *packe
 	data_ptr = ((char *)buffer) + 12;
 	memmove(data_ptr + 2, data_ptr, 26);
 
-	packet->header = packet->buffer;
-	packet->payload = (void*)((char*)packet->buffer +
-			libtrace->format->get_framing_length(packet));
+	if (legacy_prepare_packet(libtrace, packet, packet->buffer, 
+				packet->type, flags)) {
+		return -1;
+	}
 	return 68;
 }
 		
 
 static libtrace_linktype_t legacypos_get_link_type(
 		const libtrace_packet_t *packet) {
-	/* Is this a cisco hdlc frame? */
-	if ((((uint8_t*)packet->payload)[0] == 0x0F /* Unicast */
-		|| ((uint8_t*)packet->payload)[0] == 0x8F /* Multicast */)
-		&& ((uint8_t*)packet->payload)[1] == 0x00 /* control == 0x00 */
-	   )
-		return TRACE_TYPE_HDLC_POS;
-	return TRACE_TYPE_PPP;
+	/* POS can be PPP over HDLC (DLT_PPP_SERIAL), or it can be
+ 	 * just straight PPP.  Sigh.
+ 	 *
+ 	 * Ref: RFC 1549
+ 	 * 	- Perry Lorier (2008-11-04)
+ 	 */
+	if (((char *)packet->payload)[0] == '\xFF'
+		&& ((char*)packet->payload)[1] == '\x03')
+		return TRACE_TYPE_POS;
+	else 
+		return TRACE_TYPE_PPP;
 }
 
 static libtrace_linktype_t legacyatm_get_link_type(
@@ -374,7 +417,7 @@ static struct timeval legacynzix_get_timeval(const libtrace_packet_t *packet) {
 }	
 
 static void legacypos_help(void) {
-	printf("legacypos format module: $Revision: 1319 $\n");
+	printf("legacypos format module: $Revision: 1378 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacypos:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacypos:/path/to/file.gz\t(gzip-compressed)\n");
@@ -385,7 +428,7 @@ static void legacypos_help(void) {
 }
 
 static void legacyatm_help(void) {
-	printf("legacyatm format module: $Revision: 1319 $\n");
+	printf("legacyatm format module: $Revision: 1378 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyatm:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyatm:/path/to/file.gz\t(gzip-compressed)\n");
@@ -396,7 +439,7 @@ static void legacyatm_help(void) {
 }
 
 static void legacyeth_help(void) {
-	printf("legacyeth format module: $Revision: 1319 $\n");
+	printf("legacyeth format module: $Revision: 1378 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacyeth:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacyeth:/path/to/file.gz\t(gzip-compressed)\n");
@@ -407,7 +450,7 @@ static void legacyeth_help(void) {
 }
 
 static void legacynzix_help(void) {
-	printf("legacynzix format module: $Revision: 1319 $\n");
+	printf("legacynzix format module: $Revision: 1378 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tlegacynzix:/path/to/file\t(uncompressed)\n");
 	printf("\tlegacynzix:/path/to/file.gz\t(gzip-compressed)\n");
@@ -419,7 +462,7 @@ static void legacynzix_help(void) {
 
 static struct libtrace_format_t legacyatm = {
 	"legacyatm",
-	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
 	TRACE_FORMAT_LEGACY_ATM,
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -431,6 +474,7 @@ static struct libtrace_format_t legacyatm = {
 	erf_fin_input,			/* fin_input */
 	NULL,				/* fin_output */
 	legacy_read_packet,		/* read_packet */
+	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
 	legacyatm_get_link_type,	/* get_link_type */
@@ -458,7 +502,7 @@ static struct libtrace_format_t legacyatm = {
 
 static struct libtrace_format_t legacyeth = {
 	"legacyeth",
-	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
 	TRACE_FORMAT_LEGACY_ETH,
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -470,6 +514,7 @@ static struct libtrace_format_t legacyeth = {
 	erf_fin_input,			/* fin_input */
 	NULL,				/* fin_output */
 	legacy_read_packet,		/* read_packet */
+	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
 	legacyeth_get_link_type,	/* get_link_type */
@@ -497,7 +542,7 @@ static struct libtrace_format_t legacyeth = {
 
 static struct libtrace_format_t legacypos = {
 	"legacypos",
-	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
 	TRACE_FORMAT_LEGACY_POS,
 	erf_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -509,6 +554,7 @@ static struct libtrace_format_t legacypos = {
 	erf_fin_input,			/* fin_input */
 	NULL,				/* fin_output */
 	legacy_read_packet,		/* read_packet */
+	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
 	legacypos_get_link_type,	/* get_link_type */
@@ -536,7 +582,7 @@ static struct libtrace_format_t legacypos = {
 
 static struct libtrace_format_t legacynzix = {
 	"legacynzix",
-	"$Id: format_legacy.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_legacy.c 1378 2008-11-05 03:46:04Z perry $",
 	TRACE_FORMAT_LEGACY_NZIX,
 	legacynzix_init_input,		/* init_input */	
 	NULL,				/* config_input */
@@ -548,6 +594,7 @@ static struct libtrace_format_t legacynzix = {
 	erf_fin_input,			/* fin_input */
 	NULL,				/* fin_output */
 	legacynzix_read_packet,		/* read_packet */
+	legacy_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
 	legacynzix_get_link_type,	/* get_link_type */
