@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_linux.c 1319 2008-01-02 21:10:07Z perry $
+ * $Id: format_linux.c 1352 2008-05-08 04:39:14Z perry $
  *
  */
 
@@ -52,6 +52,11 @@
 
 #include <assert.h>
 
+/* Declared in linux/if_arp.h but not in net/if_arp.h sigh */
+#ifndef ARPHRD_NONE
+#define ARPHRD_NONE 0xfffe
+#endif
+
 struct tpacket_stats {
 	unsigned int tp_packets;
 	unsigned int tp_drops;
@@ -86,7 +91,7 @@ static int linuxnative_init_input(libtrace_t *libtrace)
 		malloc(sizeof(struct libtrace_format_data_t));
 	FORMAT(libtrace->format_data)->fd = -1;
 	FORMAT(libtrace->format_data)->promisc = -1;
-	FORMAT(libtrace->format_data)->snaplen = 65536;
+	FORMAT(libtrace->format_data)->snaplen = LIBTRACE_PACKET_BUFSIZE;
 	FORMAT(libtrace->format_data)->filter = NULL;
 	FORMAT(libtrace->format_data)->stats_valid = 0;
 
@@ -154,11 +159,13 @@ static int linuxnative_start_input(libtrace_t *libtrace)
 		memset(&mreq,0,sizeof(mreq));
 		mreq.mr_ifindex = addr.sll_ifindex;
 		mreq.mr_type = PACKET_MR_PROMISC;
-		setsockopt(FORMAT(libtrace->format_data)->fd,
+		if (setsockopt(FORMAT(libtrace->format_data)->fd,
 				SOL_PACKET,
 				PACKET_ADD_MEMBERSHIP,
 				&mreq,
-				socklen);
+				socklen)==-1) {
+			perror("setsockopt(PROMISC)");
+		}
 	}
 
 	if (setsockopt(FORMAT(libtrace->format_data)->fd,
@@ -330,6 +337,35 @@ static int linuxnative_config_input(libtrace_t *libtrace,
 	return -1;
 }
 
+static int linuxnative_prepare_packet(libtrace_t *libtrace, 
+		libtrace_packet_t *packet, void *buffer, 
+		libtrace_rt_types_t rt_type, uint32_t flags) {
+
+        if (packet->buffer != buffer &&
+                        packet->buf_control == TRACE_CTRL_PACKET) {
+                free(packet->buffer);
+        }
+
+        if ((flags & TRACE_PREP_OWN_BUFFER) == TRACE_PREP_OWN_BUFFER) {
+                packet->buf_control = TRACE_CTRL_PACKET;
+        } else
+                packet->buf_control = TRACE_CTRL_EXTERNAL;
+
+
+        packet->buffer = buffer;
+        packet->header = buffer;
+	packet->payload = (char *)buffer + 
+		sizeof(struct libtrace_linuxnative_header);
+	packet->type = rt_type;
+
+	if (libtrace->format_data == NULL) {
+		if (linuxnative_init_input(libtrace))
+			return -1;
+	}
+	return 0;
+	
+}
+
 #define LIBTRACE_MIN(a,b) ((a)<(b) ? (a) : (b))
 
 /* 20 isn't enough on x86_64 */
@@ -343,15 +379,18 @@ static int linuxnative_read_packet(libtrace_t *libtrace, libtrace_packet_t *pack
 	struct cmsghdr *cmsg;
 	socklen_t socklen;
 	int snaplen;
-
+	uint32_t flags = 0;
+	
 	if (!packet->buffer || packet->buf_control == TRACE_CTRL_EXTERNAL) {
 		packet->buffer = malloc((size_t)LIBTRACE_PACKET_BUFSIZE);
-		packet->buf_control = TRACE_CTRL_PACKET;
+		if (!packet->buffer) {
+			perror("Cannot allocate buffer");
+		}
 	}
 
-	packet->header = packet->buffer;
+	flags |= TRACE_PREP_OWN_BUFFER;
+	
 	packet->type = TRACE_RT_DATA_LINUX_NATIVE;
-	packet->payload = (char*)packet->buffer+sizeof(*hdr);
 
 	hdr=(struct libtrace_linuxnative_header*)packet->buffer;
 	socklen=sizeof(hdr->hdr);
@@ -369,7 +408,7 @@ static int linuxnative_read_packet(libtrace_t *libtrace, libtrace_packet_t *pack
 	msghdr.msg_controllen = CMSG_BUF_SIZE;
 	msghdr.msg_flags = 0;
 
-	iovec.iov_base = (void*)packet->payload;
+	iovec.iov_base = (void*)(packet->buffer+sizeof(*hdr));
 	iovec.iov_len = snaplen;
 
 	hdr->wirelen = recvmsg(FORMAT(libtrace->format_data)->fd, &msghdr, 0);
@@ -397,6 +436,10 @@ static int linuxnative_read_packet(libtrace_t *libtrace, libtrace_packet_t *pack
 				SIOCGSTAMP,&hdr->ts)==-1)
 		perror("ioctl(SIOCGSTAMP)");
 
+	if (linuxnative_prepare_packet(libtrace, packet, packet->buffer,
+				packet->type, flags))
+		return -1;
+	
 	return hdr->wirelen+sizeof(*hdr);
 }
 
@@ -434,6 +477,7 @@ static libtrace_linktype_t linuxnative_get_link_type(const struct libtrace_packe
 		case ARPHRD_IEEE80211:
 			return TRACE_TYPE_80211;
 		case ARPHRD_SIT:
+		case ARPHRD_NONE:
 			return TRACE_TYPE_NONE;
 		default: /* shrug, beyond me! */
 			printf("unknown Linux ARPHRD type 0x%04x\n",linktype);
@@ -489,6 +533,8 @@ static int linuxnative_get_framing_length(UNUSED
 }
 
 static int linuxnative_get_fd(const libtrace_t *trace) {
+	if (trace->format_data == NULL)
+		return -1;
 	return FORMAT(trace->format_data)->fd;
 }
 
@@ -505,14 +551,23 @@ static uint64_t linuxnative_get_filtered_packets(libtrace_t *trace) {
 
 /* Number of packets that past filtering */
 static uint64_t linuxnative_get_captured_packets(libtrace_t *trace) {
-	if (FORMAT(trace->format_data)->stats_valid != 2) {
+	if (trace->format_data == NULL)
+		return UINT64_MAX;
+	if (FORMAT(trace->format_data)->fd == -1) {
+		/* This is probably a 'dead' trace so obviously we can't query
+		 * the socket for capture counts, can we? */
+		return UINT64_MAX;
+	}
+	
+	if ((FORMAT(trace->format_data)->stats_valid & 1) 
+			|| FORMAT(trace->format_data)->stats_valid == 0) {
 		socklen_t len = sizeof(FORMAT(trace->format_data)->stats);
 		getsockopt(FORMAT(trace->format_data)->fd, 
 				SOL_PACKET,
 				PACKET_STATISTICS,
 				&FORMAT(trace->format_data)->stats,
 				&len);
-		FORMAT(trace->format_data)->stats_valid = 1;
+		FORMAT(trace->format_data)->stats_valid |= 1;
 	}
 
 	return FORMAT(trace->format_data)->stats.tp_packets;
@@ -522,21 +577,30 @@ static uint64_t linuxnative_get_captured_packets(libtrace_t *trace) {
  * of lack of space
  */
 static uint64_t linuxnative_get_dropped_packets(libtrace_t *trace) {
-	if (FORMAT(trace->format_data)->stats_valid != 1) {
+	if (trace->format_data == NULL)
+		return UINT64_MAX;
+	if (FORMAT(trace->format_data)->fd == -1) {
+		/* This is probably a 'dead' trace so obviously we can't query
+		 * the socket for drop counts, can we? */
+		return UINT64_MAX;
+	}
+	
+	if ((FORMAT(trace->format_data)->stats_valid & 2)
+			|| (FORMAT(trace->format_data)->stats_valid==0)) {
 		socklen_t len = sizeof(FORMAT(trace->format_data)->stats);
 		getsockopt(FORMAT(trace->format_data)->fd, 
 				SOL_PACKET,
 				PACKET_STATISTICS,
 				&FORMAT(trace->format_data)->stats,
 				&len);
-		FORMAT(trace->format_data)->stats_valid = 2;
+		FORMAT(trace->format_data)->stats_valid |= 2;
 	}
 
 	return FORMAT(trace->format_data)->stats.tp_drops;
 }
 
 static void linuxnative_help(void) {
-	printf("linuxnative format module: $Revision: 1319 $\n");
+	printf("linuxnative format module: $Revision: 1352 $\n");
 	printf("Supported input URIs:\n");
 	printf("\tint:\n");
 	printf("\n");
@@ -547,7 +611,7 @@ static void linuxnative_help(void) {
 }
 static struct libtrace_format_t linuxnative = {
 	"int",
-	"$Id: format_linux.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_linux.c 1352 2008-05-08 04:39:14Z perry $",
 	TRACE_FORMAT_LINUX_NATIVE,
 	linuxnative_init_input,	 	/* init_input */
 	linuxnative_config_input,	/* config_input */
@@ -559,6 +623,7 @@ static struct libtrace_format_t linuxnative = {
 	linuxnative_fin_input,		/* fin_input */
 	linuxnative_fin_output,		/* fin_output */
 	linuxnative_read_packet,	/* read_packet */
+	linuxnative_prepare_packet,	/* prepare_packet */
 	NULL,				/* fin_packet */
 	linuxnative_write_packet,	/* write_packet */
 	linuxnative_get_link_type,	/* get_link_type */
