@@ -23,7 +23,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: format_tsh.c 1319 2008-01-02 21:10:07Z perry $
+ * $Id: format_tsh.c 1375 2008-10-28 04:34:47Z perry $
  *
  */
 #include "config.h"
@@ -49,8 +49,7 @@ struct tsh_format_data_t {
 
 typedef struct tsh_pkt_header_t {
 	uint32_t seconds;
-	LT_BITFIELD32 interface:8;
-	LT_BITFIELD32 usecs:24;
+	uint32_t usecs;
 } tsh_pkt_header_t;
 
 static int tsh_get_framing_length(const libtrace_packet_t *packet)
@@ -88,13 +87,39 @@ static int tsh_fin_input(libtrace_t *libtrace) {
 	return 0;
 }
 
+static int tsh_prepare_packet(libtrace_t *libtrace, libtrace_packet_t *packet,
+		void *buffer, libtrace_rt_types_t rt_type, uint32_t flags) {
+	if (packet->buffer != buffer &&
+                        packet->buf_control == TRACE_CTRL_PACKET) {
+                free(packet->buffer);
+        }
+
+        if ((flags & TRACE_PREP_OWN_BUFFER) == TRACE_PREP_OWN_BUFFER) {
+                packet->buf_control = TRACE_CTRL_PACKET;
+        } else
+                packet->buf_control = TRACE_CTRL_EXTERNAL;
+
+
+        packet->buffer = buffer;
+        packet->header = buffer;
+	packet->type = rt_type;
+	packet->payload = (char *)packet->buffer + sizeof(tsh_pkt_header_t);
+
+	if (libtrace->format_data == NULL) {
+		if (tsh_init_input(libtrace))
+			return -1;
+	}
+
+	return 0;
+}
+
 static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	int numbytes;
 	void *buffer2 = packet->buffer;
+	uint32_t flags = 0;
 
 	if (!packet->buffer || packet->buf_control == TRACE_CTRL_EXTERNAL) {
 		packet->buffer = malloc((size_t)LIBTRACE_PACKET_BUFSIZE);
-		packet->buf_control = TRACE_CTRL_PACKET;
 		if (!packet->buffer) {
 			trace_set_err(libtrace, errno, 
 					"Cannot allocate memory");
@@ -102,7 +127,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		}
 	}
 
-	packet->header = packet->buffer;
+	flags |= TRACE_PREP_OWN_BUFFER;
 	packet->type = TRACE_RT_DATA_TSH;
 
 	buffer2 = packet->buffer;
@@ -120,8 +145,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 		return 0;
 	}
 
-	buffer2+=numbytes;
-	packet->payload = buffer2;
+	buffer2 = (char*)buffer2 + numbytes;
 
 	/* Read the IP header */
 	if ((numbytes=libtrace_io_read(DATA(libtrace)->file,
@@ -136,7 +160,7 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 	/* IP Options aren't captured in the trace, so leave room
 	 * for them, and put the transport header where it "should" be
 	 */
-	buffer2+=((libtrace_ip_t*)buffer2)->ip_hl*4;
+	buffer2 = (char*)buffer2 + ((libtrace_ip_t*)buffer2)->ip_hl*4;
 
 	/* Read the transport header */
 	if ((numbytes=libtrace_io_read(DATA(libtrace)->file,
@@ -146,6 +170,12 @@ static int tsh_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet) {
 				libtrace->uridata);
 		return -1;
 	}
+	
+	if (tsh_prepare_packet(libtrace, packet, packet->buffer, packet->type, 
+				flags)) {
+		return -1;
+	}
+
 
 	return 80;
 }
@@ -155,14 +185,14 @@ static libtrace_linktype_t tsh_get_link_type(const libtrace_packet_t *packet) {
 }
 
 static libtrace_direction_t tsh_get_direction(const libtrace_packet_t *packet) {
-	return ((tsh_pkt_header_t*)(packet->header))->interface;
+	return ntohl(((tsh_pkt_header_t*)(packet->header))->usecs & htonl(0xFF000000)) >> 24;
 }
 
 static struct timeval tsh_get_timeval(const libtrace_packet_t *packet)
 {
 	struct timeval tv;
 	tv.tv_sec=ntohl(((tsh_pkt_header_t*)(packet->header))->seconds);
-	tv.tv_usec=ntohl(((tsh_pkt_header_t*)(packet->header))->usecs);
+	tv.tv_usec=ntohl(((tsh_pkt_header_t*)(packet->header))->usecs & htonl(0x00FFFFFF));
 
 	return tv;
 }
@@ -179,7 +209,7 @@ static int tsh_get_wire_length(const libtrace_packet_t *packet) {
 }
 
 static void tsh_help(void) {
-	printf("tsh format module: $Revision: 1319 $\n");
+	printf("tsh format module: $Revision: 1375 $\n");
 	printf("Supported input URIs:\n");
 	printf("\ttsh:/path/to/file\t(uncompressed)\n");
 	printf("\ttsh:/path/to/file.gz\t(gzip-compressed)\n");
@@ -192,7 +222,7 @@ static void tsh_help(void) {
 
 static struct libtrace_format_t tshformat = {
 	"tsh",
-	"$Id: format_tsh.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_tsh.c 1375 2008-10-28 04:34:47Z perry $",
 	TRACE_FORMAT_TSH,
 	tsh_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -204,6 +234,7 @@ static struct libtrace_format_t tshformat = {
 	tsh_fin_input,			/* fin_input */
 	NULL,				/* fin_output */
 	tsh_read_packet,		/* read_packet */
+	tsh_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
 	tsh_get_link_type,		/* get_link_type */
@@ -236,7 +267,7 @@ static struct libtrace_format_t tshformat = {
  */
 static struct libtrace_format_t frplusformat = {
 	"fr+",
-	"$Id: format_tsh.c 1319 2008-01-02 21:10:07Z perry $",
+	"$Id: format_tsh.c 1375 2008-10-28 04:34:47Z perry $",
 	TRACE_FORMAT_TSH,
 	tsh_init_input,			/* init_input */	
 	NULL,				/* config_input */
@@ -248,6 +279,7 @@ static struct libtrace_format_t frplusformat = {
 	tsh_fin_input,			/* fin_input */
 	NULL,				/* fin_output */
 	tsh_read_packet,		/* read_packet */
+	tsh_prepare_packet,		/* prepare_packet */
 	NULL,				/* fin_packet */
 	NULL,				/* write_packet */
 	tsh_get_link_type,		/* get_link_type */
