@@ -85,7 +85,9 @@
     #define LT_BITFIELD32 	uint32_t
     #define LT_BITFIELD64 	uint64_t
 #else
-#   include <stdint.h>
+    #ifdef HAVE_STDINT_H
+	#   include <stdint.h>
+    #endif
     #ifdef HAVE_GCCVISIBILITYPATCH
 	#define DLLEXPORT __attribute__ (visibility("default"))
 	#define DLLLOCAL __attribute__ (visibility("hidden"))
@@ -128,10 +130,10 @@
 
 /** API version as 2 byte hex digits, eg 0xXXYYZZ */
 #define LIBTRACE_API_VERSION \
-            ((3<<16)|(0<<8)|(3))
+            ((3<<16)|(0<<8)|(6))
 
 #define LIBTRACE_SVN_REVISION 0
-#define DAG_DRIVER_V ""
+#define DAG_DRIVER_V "DAG not present"
     
 #ifdef __cplusplus 
 extern "C" { 
@@ -219,6 +221,7 @@ typedef enum {
 #else
 	TRACE_DLT_RAW = 12,
 #endif
+	TRACE_DLT_PPP_SERIAL = 50,
 	TRACE_DLT_LINKTYPE_RAW = 101,
 	TRACE_DLT_C_HDLC = 104,
 	TRACE_DLT_IEEE802_11 = 105,
@@ -303,6 +306,7 @@ typedef enum {
 	TRACE_RT_DATA_LEGACY_POS=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_LEGACY_POS, 
 	TRACE_RT_DATA_LEGACY_ETH=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_LEGACY_ETH, 
 	TRACE_RT_DATA_LINUX_NATIVE=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_LINUX_NATIVE,
+	TRACE_RT_DATA_BPF       =TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_BPF,
 	TRACE_RT_DATA_TSH	=TRACE_RT_DATA_SIMPLE+TRACE_FORMAT_TSH,
 
 	TRACE_RT_DATA_ATMHDR = TRACE_RT_DATA_SIMPLE + TRACE_FORMAT_ATMHDR,
@@ -575,13 +579,27 @@ typedef struct libtrace_atm_nni_capture_cell
 } PACKED libtrace_atm_nni_capture_cell_t;
 
 /** PPP header */
-typedef struct libtrace_pps
+typedef struct libtrace_ppp
 {
- uint8_t addres;		/**< PPP Address (0xFF - All stations) */
- uint8_t header;		/**< PPP Control (0x03 - Unnumbered info) */
+ /* I can't figure out where the hell these two variables come from. They're
+  * definitely not in RFC 1661 which defines PPP. Probably some weird thing
+  * relating to the lack of distinction between PPP, HDLC and CHDLC */
+	
+/* uint8_t address; */		/**< PPP Address (0xFF - All stations) */
+/* uint8_t header;  */		/**< PPP Control (0x03 - Unnumbered info) */
  uint16_t protocol;		/**< PPP Protocol (htons(0x0021) - IPv4 */
 } PACKED libtrace_ppp_t;
 
+/** PPPoE header */
+typedef struct libtrace_pppoe
+{
+ LT_BITFIELD8 version:4;	/**< Protocol version number */
+ LT_BITFIELD8 type:4;		/**< PPPoE Type */
+ uint8_t code;			/**< PPPoE Code */
+ uint16_t session_id;		/**< Session Identifier */
+ uint16_t length;		/**< Total Length of the PPP packet */
+} PACKED libtrace_pppoe_t;
+
 /** 802.11 header */
 typedef struct libtrace_80211_t {
 #if BYTE_ORDER == LITTLE_ENDIAN
@@ -1059,6 +1077,9 @@ DLLEXPORT void *trace_get_packet_meta(const libtrace_packet_t *packet,
  * Examples of metadata headers include the Linux SLL header and the Radiotap
  * wireless monitoring header.
  *
+ * If the header is truncated, this function will return NULL, and remaining will be 0.
+ * If there are 0 bytes of payload, the function will return a pointer, and remaining will be 0.
+ *
  * @param[in] meta a pointer to a header
  * @param[in,out] linktype the linktype of meta (updated to indicate the
  * linktype of the returned header if applicable).
@@ -1067,7 +1088,7 @@ DLLEXPORT void *trace_get_packet_meta(const libtrace_packet_t *packet,
  * pointer to a metadata header, NULL is returned and linktype remains
  * unchanged.
  *
- * All parameters are mandatory. NULL will be returned if any are NULL.
+ * All parameters are mandatory. 
  */
 DLLEXPORT void *trace_get_payload_from_meta(const void *meta,
                 libtrace_linktype_t *linktype,
@@ -1098,17 +1119,19 @@ DLLEXPORT void *trace_get_layer2(const libtrace_packet_t *packet,
  *
  * @param l2            	The pointer to the current layer2 header
  * @param linktype		The type of the layer2 header
- * @param[out] ethertype 	An optional output variable of the ethernet type
- * @param[in,out] remaining 	Optionally updated with the length remaining
+ * @param[out] ethertype 	An optional output variable of the ethernet type of the new header
+ * @param[in,out] remaining 	Updated with the length remaining
  *
  * @return a pointer to the transport layer header, or NULL if header isn't
  * present.
  *
- * type may be NULL if not needed.
+ * Remaining must point to the number of bytes captured of the layer2 header
+ * and beyond.  It will be decremented by the number of bytes skipped to find
+ * the payload.
  *
- * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the layer2 header and beyond.  It will be decremented
- * by the number of bytes skipped to find the payload.
+ * If the layer2 header is complete but there are zero bytes of payload after the end of the header,
+ * a pointer to where the payload would be is returned and remaining will be set to 0.  If the 
+ * layer2 header is incomplete (truncated), then NULL is returned and remaining will be set to 0.
  *
  */
 DLLEXPORT void *trace_get_payload_from_layer2(void *l2,
@@ -1151,11 +1174,12 @@ DLLEXPORT void *trace_get_transport(const libtrace_packet_t *packet,
  * @return a pointer to the transport layer header, or NULL if header isn't
  * present.
  *
- * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the IP header and beyond.  It will be decremented by
- * the length of the IPv4 header (including any options).
+ * Remaining will be decremented by the size of the IPv4 header (including any options). 
+ * If the IPv4 header is complete but there are zero bytes of payload after the IPv4 header, a pointer
+ * to where the payload would be is returned and remaining will be set to 0.  If the IPv4 header is
+ * incomplete, NULL will be returned and remaining will be set to 0.
  *
- * proto may be NULL if not needed.
+ * proto may be NULL, in which case it won't be updated
  *
  * @note This is similar to trace_get_transport_from_ip in libtrace2
  */
@@ -1170,11 +1194,12 @@ DLLEXPORT void *trace_get_payload_from_ip(libtrace_ip_t *ip, uint8_t *proto,
  * @return a pointer to the transport layer header, or NULL if the IPv6 header
  * isn't complete.
  *
- * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the IP header and beyond.  It will be decremented by
- * this function by the length of the IPV6 header.
+ * Remaining will be decremented by the size of the IPv6 header (including any options). 
+ * If the IPv6 header is complete but there are zero bytes of payload after the IPv6 header, a pointer
+ * to where the payload would be is returned and remaining will be set to 0.  If the IPv6 header is
+ * incomplete, NULL will be returned and remaining will be set to 0.
  *
- * proto may be NULL if not needed.
+ * proto may be NULL, in which case it won't be updated.
  *
  */
 DLLEXPORT void *trace_get_payload_from_ip6(libtrace_ip6_t *ipptr,
@@ -1200,35 +1225,64 @@ DLLEXPORT void *trace_get_payload_from_link(void *linkptr,
 		libtrace_linktype_t linktype, 
 		uint16_t *type, uint32_t *remaining);
 
-/** Skips over any 802.1q headers, if present.
- * @param ethernet      A pointer to the payload following an ethernet header
- * -usually the result of calling trace_get_payload_from_link
- * @param[in,out] type  The ethernet type, replaced with the vlan ether type
+/** Skips over any 802.1q header, if present.
+ * @param vlan      A pointer to the vlan header
+ * @param[out] type  The ethernet type, replaced with the vlan ether type
  * @param[in,out] remaining Updated with the number of bytes remaining
  *
  * @return a pointer to the header beyond the vlan header, if present.
- * Otherwise, returns the ethernet payload that was passed in
+ * Otherwise, returns NULL.  
  *
- * Remaining may be NULL. If Remaining is not NULL it must point to the number
- * of bytes captured past (but not including) the link layer. This function
- * will decrement it by the length of the 802.1q headers if present.
+ * Remaining will be decremented by the size of the vlan header.  If the vlan
+ * header is complete but there are zero bytes of payload after the vlan
+ * header, a pointer to where the payload would be is returned and remaining
+ * will be set to 0.  If the vlan header is incomplete, NULL will be returned
+ * and remaining will be set to 0.
+ *
+ * type will be set to the ethertype contained within the vlan payload.
+ *
+ */
+DLLEXPORT void *trace_get_payload_from_vlan(
+                void *vlan, uint16_t *type, uint32_t *remaining);
+
+/** Skips over a MPLS header
+ * @param mpls      A pointer to the mpls header
+ * @param[out] type  The ethernet type, replaced by the ether type of the
+ * following header - 0x0000 if an Ethernet header is deemed to be next
+ * @param[in,out] remaining     Updated with the number of bytes remaining
+ *
+ * @return a pointer to the header beyond the MPLS label, if present. Will
+ * return NULL if there is not enough bytes remaining to skip past the MPLS
+ * label.
+ *
+ * Remaining will be decremented by the size of the MPLS header.  If the MPLS
+ * header is complete but there are zero bytes of payload after the MPLS
+ * header, a pointer to where the payload would be is returned and remaining
+ * will be set to 0.  If the MPLS header is incomplete, NULL will be returned
+ * and remaining will be set to 0.
  *
  * Type must point to the value of the ethernet type. Libtrace will assert
  * fail if type is NULL.
  *
+ * NOTE that this function will only remove one MPLS label at a time - the type
+ * will be set to 0x8847 if there is another MPLS label after the one
+ * removed by this function.
+ *
  */
-DLLEXPORT void *trace_get_vlan_payload_from_ethernet_payload(
-                void *ethernet_payload, uint16_t *type, uint32_t *remaining);
+DLLEXPORT void *trace_get_payload_from_mpls(
+                void *mpls, uint16_t *type, uint32_t *remaining);
 
 /** Gets a pointer to the payload given a pointer to a tcp header
  * @param tcp           The tcp Header
  * @param[in,out] remaining Updated with the number of bytes remaining
  *
- * @return a pointer to the tcp payload, or NULL if the payload isn't present.
+ * @return a pointer to the tcp payload, or NULL if the tcp header is truncated.
  *
- * Remaining may be NULL.  If remaining is not NULL it must point to the number
- * of bytes captured of the TCP header and beyond.  It will be decremented by
- * this function by the length of the TCP header (including any options).
+ * Remaining will be decremented by the size of the TCP header.  If the TCP
+ * header is complete but there are zero bytes of payload after the TCP
+ * header, a pointer to where the payload would be is returned and remaining
+ * will be set to 0.  If the TCP header is incomplete, NULL will be returned
+ * and remaining will be set to 0.
  *
  */
 DLLEXPORT void *trace_get_payload_from_tcp(libtrace_tcp_t *tcp, 
@@ -1238,11 +1292,13 @@ DLLEXPORT void *trace_get_payload_from_tcp(libtrace_tcp_t *tcp,
  * @param udp           The udp Header
  * @param[in,out] remaining Updated with the number of bytes remaining
  *
- * @return a pointer to the udp payload, or NULL if the payload isn't present.
+ * @return a pointer to the udp payload, or NULL if the udp header is truncated.
  *
- * Remaining may be NULL.  If Remaining is not NULL it must point to the number
- * of bytes captured of the UDP header and beyond.  It will be decremented by
- * this function to the number of bytes remaining after the UDP header.
+ * Remaining will be decremented by the size of the TCP header.  If the TCP
+ * header is complete but there are zero bytes of payload after the TCP
+ * header, a pointer to where the payload would be is returned and remaining
+ * will be set to 0.  If the TCP header is incomplete, NULL will be returned
+ * and remaining will be set to 0.
  *
  */
 DLLEXPORT void *trace_get_payload_from_udp(libtrace_udp_t *udp, uint32_t *remaining);
@@ -1251,11 +1307,13 @@ DLLEXPORT void *trace_get_payload_from_udp(libtrace_udp_t *udp, uint32_t *remain
  * @param icmp          The icmp Header
  * @param[in,out] remaining Updated with the number of bytes remaining
  *
- * @return a pointer to the icmp payload, or NULL if the payload isn't present.
+ * @return a pointer to the icmp payload, or NULL if the icmp header is truncated.
  *
- * Remaining may be NULL.  If remaining is not NULL it must point to the number
- * of bytes captured of the ICMP header and beyond.  It will be decremented
- * by the number of bytes in the ICMP header.
+ * Remaining will be decremented by the size of the TCP header.  If the TCP
+ * header is complete but there are zero bytes of payload after the TCP
+ * header, a pointer to where the payload would be is returned and remaining
+ * will be set to 0.  If the TCP header is incomplete, NULL will be returned
+ * and remaining will be set to 0.
  * 
  */
 DLLEXPORT void *trace_get_payload_from_icmp(libtrace_icmp_t *icmp, 
