@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: trace.c 1319 2008-01-02 21:10:07Z perry $
+ * $Id: trace.c 1367 2008-08-22 01:25:59Z perry $
  *
  */
 
@@ -547,6 +547,11 @@ DLLEXPORT int trace_config(libtrace_t *libtrace,
 			if (trace_is_err(libtrace)) {
 				err = trace_get_err(libtrace);
 			}
+			if (*(int*)value<0 
+				|| *(int*)value>LIBTRACE_PACKET_BUFSIZE) {
+				trace_set_err(libtrace,TRACE_ERR_BAD_STATE,
+					"Invalid snap length");
+			}
 			libtrace->snaplen=*(int*)value;
 			return 0;
 		case TRACE_OPTION_FILTER:
@@ -621,6 +626,8 @@ DLLEXPORT void trace_destroy(libtrace_t *libtrace) {
 
 DLLEXPORT void trace_destroy_dead(libtrace_t *libtrace) {
 	assert(libtrace);
+	if (libtrace->format_data)
+		free(libtrace->format_data);
 	free(libtrace);
 }
 /* Close an output trace file, freeing up any resources it may have been using
@@ -726,14 +733,15 @@ DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 		libtrace->format->fin_packet(packet);
 	}
 
-	/* Clear the packet cache */
-	packet->capture_length = -1;
-	packet->l3_header = NULL;
-	packet->l3_ethertype = 0;
 
 	if (libtrace->format->read_packet) {
 		do {
 			size_t ret;
+			/* Clear the packet cache */
+			packet->capture_length = -1;
+			packet->l3_header = NULL;
+			packet->l3_ethertype = 0;
+
 			ret=libtrace->format->read_packet(libtrace,packet);
 			if (ret==(size_t)-1 || ret==0) {
 				return ret;
@@ -760,6 +768,38 @@ DLLEXPORT int trace_read_packet(libtrace_t *libtrace, libtrace_packet_t *packet)
 	return ~0U;
 }
 
+int trace_prepare_packet(libtrace_t *trace, libtrace_packet_t *packet,
+		void *buffer, libtrace_rt_types_t rt_type, uint32_t flags) {
+
+	assert(packet);
+	assert(trace);
+	
+	/* XXX Proper error handling?? */
+	if (buffer == NULL)
+		return -1;
+
+	if (!(packet->buf_control==TRACE_CTRL_PACKET || packet->buf_control==TRACE_CTRL_EXTERNAL)) {
+		trace_set_err(trace,TRACE_ERR_BAD_STATE,"Packet passed to trace_read_packet() is invalid\n");
+		return -1;
+	}
+	
+	packet->trace = trace;
+	
+	/* Clear packet cache */
+	packet->capture_length = -1;
+	packet->l3_header = NULL;
+	packet->l3_ethertype = 0;
+
+	if (trace->format->prepare_packet) {
+		return trace->format->prepare_packet(trace, packet,
+				buffer, rt_type, flags);
+	}
+	trace_set_err(trace, TRACE_ERR_UNSUPPORTED, 
+			"This format does not support preparing packets\n");
+	return -1;
+
+}
+
 /* Writes a packet to the specified output
  *
  * @param libtrace	describes the output format, destination, etc.
@@ -815,13 +855,14 @@ DLLEXPORT uint64_t trace_get_erf_timestamp(const libtrace_packet_t *packet) {
 	} else if (packet->trace->format->get_timeval) {
 		/* timeval -> timestamp */
 		ts = packet->trace->format->get_timeval(packet);
-		timestamp = ((((uint64_t)ts.tv_sec) << 32) + \
-				(((uint64_t)ts.tv_usec * UINT_MAX)/1000000));
+		timestamp = ((((uint64_t)ts.tv_sec) << 32) +
+				(((uint64_t)ts.tv_usec << 32)/1000000));
 	} else if (packet->trace->format->get_seconds) {
 		/* seconds -> timestamp */
 		seconds = packet->trace->format->get_seconds(packet);
-		timestamp = ((uint64_t)((uint32_t)seconds) << 32) + \
-		      (uint64_t)(( seconds - (uint32_t)seconds   ) * UINT_MAX);
+		timestamp = (((uint64_t)seconds)<<32)
+		          + (uint64_t)((seconds-(uint64_t)seconds)*UINT_MAX);
+		      
 	}
 	return timestamp;
 }
@@ -906,6 +947,8 @@ DLLEXPORT size_t trace_get_capture_length(const libtrace_packet_t *packet)
 			packet->trace->format->get_capture_length(packet);
 	}
 
+	assert(packet->capture_length < LIBTRACE_PACKET_BUFSIZE);
+
 	return packet->capture_length;
 }
 	
@@ -1594,6 +1637,7 @@ void trace_construct_packet(libtrace_packet_t *packet,
 	packet->type=pcap_linktype_to_rt(libtrace_to_pcap_linktype(linktype));
 }
 
+
 uint64_t trace_get_received_packets(libtrace_t *trace)
 {
 	assert(trace);
