@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: tracertstats.c 1270 2007-09-05 23:53:07Z perry $
+ * $Id: tracertstats.c 1384 2008-11-26 21:59:46Z perry $
  *
  */
 
@@ -103,7 +103,12 @@ static void run_trace(char *uri)
 	double last_ts = 0;
 	double ts = 0;
 
+	fprintf(stderr,"output format: '%s'\n",output_format);
 	output=output_init(uri,output_format?output_format:"txt");
+	if (!output) {
+		fprintf(stderr,"Failed to create output file\n");
+		return;
+	}
 	output_add_column(output,"ts");
 	output_add_column(output,"packets");
 	output_add_column(output,"bytes");
@@ -133,11 +138,9 @@ static void run_trace(char *uri)
 
         for (;;) {
 		int psize;
-		dag_record_t *erf_hdr;
                 if ((psize = trace_read_packet(trace, packet)) <1) {
                         break;
                 }
-		erf_hdr = (dag_record_t *)packet->header;
 		
 		if (trace_get_packet_buffer(packet,NULL,NULL) == NULL) {
 			continue;
