@@ -24,7 +24,7 @@
  * along with libtrace; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: tracestats.c 1310 2007-12-03 10:11:00Z perry $
+ * $Id: tracestats.c 1339 2008-04-02 03:34:39Z spa1 $
  *
  */
 
@@ -53,10 +53,7 @@
 #include <signal.h>
 
 #include "libtrace.h"
-
-#ifndef PRIu64
-	#define PRIu64 "llu"
-#endif
+#include "lt_inttypes.h"
 
 struct libtrace_t *trace;
 
